#  Code for generating distribution of model parameters for Sobol sensitivity 
#  indices and confidence interval calculations. 

#  Number of parameter sets to generate 
nparsets<-1001

#  Parameter Distributions:
#	p4pars:  truncated logistic distribution parameters (alpha and beta) for p4
#	etapars:  mean and variance of log(parr production)
#	gammapars:  regression parameters for parr-to-adult survival 
#	nupars:	regression parameters for smolt-to-adult survival
#  Store point estimates with same name + PE

#----------------------------------------------  Age-at-spawning distribution  ---------------------------------------
H<-read.csv("c:\\viability\\ecology_sf\\sdata.csv");
northfork<-(stock=="Bear")|(stock=="Marsh")|(stock=="Sulphur");
H<-H[northfork,];
p4<-H$AGE4/(H$AGE4+H$AGE5); 
p4t<-p4[(H$AGE4+H$AGE5)>=5]; # only include years with at least 5 spawners

#-Fit truncated logistic model to p4t values
dTlogis<-function(x,m,s) {dlogis(x,m,s)/(plogis(1,m,s)-plogis(0,m,s))}
dTlik<-function(pars) {-sum(log(dTlogis(p4t,pars[1],pars[2])))}
sfit<-optim(c(0.05,0.2),dTlik,method="Nelder-Mead",control=list(trace=0,parscale=c(100,100))); 
sfit2<-optim(sfit$par,dTlik,method="Nelder-Mead",control=list(trace=0,parscale=c(100,100))); 
sfit$par; sfit2$par; p4pars<-sfit2$par; 
p4parsPE<-p4pars; #  Point Estimates for p4 parameters

#-Fit truncated logistic model to bootstrap samples of p4t values
p4pars<-matrix(0,nparsets,2)

for (i in 1:nparsets){
p4tboot<-sample(p4t,replace=T); 
dTbootlik<-function(pars) {-sum(log(dTlogis(p4tboot,pars[1],pars[2])))}
bootfit<-optim(c(0.05,0.2),dTbootlik,method="Nelder-Mead",control=list(trace=0,parscale=c(100,100))); 
bootfit2<-optim(bootfit$par,dTbootlik,method="Nelder-Mead",control=list(trace=0,parscale=c(100,100))); 
p4pars[i,]<-bootfit2$par
}


# ------------------ Parr Production:  use parametric bootstap w/ lognormal distribution --------------------------
#  Read in the data from Paulsen and Fisher (2001) updated for 1999 and 2000
P <-read.csv("c:\\viability\\ecology_sf\\paulsen_sf.csv");
P <-na.omit(P)

#  Log smolt-per-spawner ratios for broods 1990-1997 (smolt years 1992-1999) from  Petrosky et al. (2001)
lnEt<-c(4.09, 4.69, 3.71, 4.59, 5.06, 4.73, 5.15, 4.50, 4.38)

#  Estimate yearly means for the survival rates in P for 1992-1999
Ptemp<-P[P$TAGYEAR!="2000",] 

#parr-to-smolt survival rates, unstuck by year
temp<-unstack(Ptemp$surv2, Ptemp$surv2~Ptemp$TAGYEAR) 
rm(Ptemp)

z<-sapply(temp,mean) # gives the years means across the three stocks


#  Estimated mean and variance of eta_t on log scale from combining the two data sets
meanP <- mean(lnEt)-mean(log(z));  
varP <- var(lnEt)-var(log((z)))
etaparsPE<-cbind(meanP, varP)

rm(lnEt)
rm(z)

#  Generate nparsets parametric bootstraps of to give mean and variance
#  of eta_t

temp<-matrix(rnorm(nparsets*8,meanP, sqrt(varP)),8,nparsets)
#  Bootstrap estimates of mean and variance of eta_t
etapars<- cbind(apply(temp,2,mean), apply(temp,2,var))



#--------Parr-to-smolt survival and D values --------------------
 
#  Fit mixed model to survival rates from Bear Valley Creak, Elk Creek 
#  and Sulfur Creek

temp <- P[(P$site=="BEARVC" |P$site=="ELKC") |(P$site=="SULFUC"),]

#  Form Drought indices
Drought<- rep(0,dim(temp)[1])
Drought[temp$pdsi < -3]<-1
temp <- cbind(temp, Drought)

#  Fit logit survival model
temp2<-lm(log(surv2/(1-surv2))~Drought+factor(TAGYEAR),data=temp)
gammaregsPE <-coef(temp2)[2];  # coef for Drought
gammayreffectsPE<- c(coef(temp2)[1], coef(temp2)[1]+coef(temp2)[3:7])
gammaresidsPE<- temp2$residuals
 
#  Regression boostrap
gammaregs<-matrix(0,nparsets, 1)
gammayreffects<-matrix(0,nparsets,6)
gammaresids<-matrix(0,nparsets,18)

for (i in 1:nparsets){

#  Create bootstrap dataset
z <-predict(temp2)+sample(resid(temp2), replace=T); 
z2<-temp
z2<-cbind(z2,z) 

# fit survival model to bootstrap data set
temp3<-lm(z~Drought+factor(TAGYEAR), data=z2)
gammaregs[i,] <-coef(temp3)[2]
gammayreffects[i,]<- c(coef(temp3)[1], coef(temp3)[1]+coef(temp3)[3:7])
gammaresids[i,]<- temp3$residuals

}
gammapars<-list(g1=gammaregs, g2=gammayreffects, g3=gammaresids)
gammaparsPE<-list(g1=gammaregsPE, g2=gammayreffectsPE, g3=gammaresidsPE)


#-------------  Smolt-to-adult Survival from Levin et al ----------
#  Fit analysis of covariace model
#  params = b0-b3 (regression parameters) and sigma 
nupars<-matrix(0,nparsets,5)

#  Read Levin data set
L <-read.csv("c:\\viability\\ecology_sf\\levinsurv.csv");
logits<-log(L$surv/(1-L$surv))
L<-cbind(L, logits) 

nparsPE<-matrix(0,5,1)
temp<-lm(logits~Release+Good+Release*Good, data=L)
nparsPE[1:4]<-coef(temp)
nparsPE[5]<-sd(resid(temp))
 
for (i in 1:nparsets){
    logitstemp<-  nparsPE[1]+nparsPE[2]*L$Release+nparsPE[3]*L$Good+nparsPE[4]*L$Release*L$Good+rnorm(20,0, nparsPE[5]);
    temp<-lm(logitstemp~L$Release+L$Good+L$Release*L$Good)
    nupars[i,1:4]<-coef(temp);
    nupars[i,5]<-sd(resid(temp));
}



