################################################################## 
# This file contains the statistical analyses behind the         
# models for the distributions of Et and p4t for the		
# unstructured models. As noted in the paper, we considered	
# a number of models for p4t and only the final one is		
# presented here (truncated logistic distribution).   		
#
# The purpose of this file is to fully document 	
# the procedures described in the published paper. 
# We are not at liberty to distribute the data on which the	
# analyses were based, so users will not be able to replicate	
# our analyses. 
##################################################################


#------------------ Load and select the data 

####### Read in the complete data file
H<-read.csv("e:\\projects\\UncertaintyFeature\\sdata2.csv");

###### Remove missing values
e<-H$AGE4>=0; H<-H[e,];

###### Select the stocks to be modeled
stock<-H$STOCK;
northfork<-(stock=="Bear")|(stock=="Marsh")|(stock=="Sulphur");
H<-H[northfork,];

###### unpack the variables into separate vectors
year<-H$YEAR;
spawners<-H$SPAWNERS; 
Et<-(H$AGE4+H$AGE5)/spawners;
p4<-H$AGE4/(H$AGE4+H$AGE5); 


#----------------- Select and fit model for Et ------------------#

###### First, look at the data 
plot(year,Et);

###### Remove an outlier and create data frame for model fitting
e<-Et>0; 
sdata<-data.frame(year=year[e], fyear=factor(year[e]),spawners=spawners[e],
Et=Et[e],stock=H$STOCK[e],p4=p4[e],recruits=H$AGE4[e]+H$AGE5[e]); 


##### First, some general models for Et and pt 
Emodel1<-aov(log(Et)~fyear+stock,data=sdata);
Emodel2<-aov(log(Et)~year*stock,data=sdata); 
summary(Emodel1); 
summary(Emodel2)
# Et has a year and fyear effects, P<0.001, but stock and stock*year
# 	effects are nonsignificant (P>0.1); 

cor.test(sdata$Et,sdata$p4,method="k");
# p4 and Et appear to be independent. 

###### Now model the year effect in Et 

###### Spline fit indicates leveling off in Et after 1984 
library(mgcv); 
mm<-gam(log(1+Et)~s(year)); plot(mm);

###### Omit early years and data with very few spawners
end<- year>83&spawners>20; 

###### Create a data frame, adding a factor variable for year (fyear)
###### and a factor variable byear to indicate a "bad year" 

enddata<-data.frame(year=year[end], fyear=factor(year[end]),
byear=factor((year[end]==90)|(year[end]==91)),spawners=spawners[end],
Et=Et[end],stock=H$STOCK[end],p4=p4[end],recruits=H$AGE4[end]+H$AGE5[end]); 

##### Try Et again, with the early years omitted 
library(MASS);
Emodel<-aov(Et~fyear,data=enddata); 
boxcox(Emodel); 
# optimal transformation is close to lambda=0 (log transform) and
# 95% CI overlaps 0 ==> reasonable to use log transform. 

Emodel1<-aov(log(Et)~year+stock,data=enddata);
Emodel2<-aov(log(Et)~fyear+stock,data=enddata); 
summary(Emodel1); 
# Post-1984 there is is no trend over time in Et
summary(Emodel2);
# but there still is a year effect
# In both models, the stock effect is insignificant  

Emodel3<-aov(log(Et)~fyear,data=enddata); 
Emodel4<-aov(log(Et)~byear,data=enddata); 
anova(Emodel4,Emodel3);
### Separate year effects (rather than good/bad) not supported

## compute and plot all annual means: this makes Figure 1
years<-unique(sdata$year); 
yij<-log(sdata$Et); 
nyr<-length(years);
ybari<-rep(0,nyr);
for (i in 1:nyr) {
	e<-sdata$year==years[i];
	ybari[i]<-mean(yij[e]);
}

par(cex.lab=1.75,cex.axis=1.75,mar=c(7,7,7,7),las=1);
plot(years,ybari,type="o",pch=15,xlab="Year",
ylab="Mean log spawner production",
cex=1.75); 

#----------------- Select and fit model for p4t ------------------#
###### omit p4 estimates based on few spawners
p4t<-p4[(H$AGE4+H$AGE5)>=10];
s5<-(H$AGE4+H$AGE5)>=10

######################## year or stock effects?
Pmodel0<-lm(0.001+p4[s5]~fyear[s5]+stock[s5],data=sdata); 
library(MASS); 
boxcox(Pmodel0,plotit=T); 
# boxcox indicates sqrt transform for normality 

Pmodel1<-aov(sqrt(p4[s5])~fyear[s5]+stock[s5],data=sdata); 
summary(Pmodel1); #year and stock effects insignificant

##### See if stock effect is significant if year is removed & vice versa
Pmodel2<-aov(sqrt(p4[s5])~fyear[s5],data=sdata); 
Pmodel3<-aov(sqrt(p4[s5])~stock[s5],data=sdata); 
summary(Pmodel2); summary(Pmodel3); 
# year effect is marginal but in 
Pmodel3<-lm(sqrt(p4[s5])~fyear[s5],data=sdata); 
summary(Pmodel3); 
# the 1989 coefficient is highly significant. 

###### Do we believe that? With so many explanatory variables it's a 
# "fishing expedition". 
# Simulation: randomize the response and see if
# we still get a significant year effect for some year
x<-sdata$fyear[s5]; y<-sqrt(sdata$p4[s5]);
pvals<-matrix(0,500,1); 
for (j in 1:500) {
	yj<-sample(y); 
	mj<-summary(lm(yj~x))$coefficients;
	mj<-mj[-1,4];
	cat(j,min(mj),"\n");
	pvals[j]<-min(mj);
}
# mean(pvals<0.05)=0.41; mean(pvals<0.01)=0.086
# A highly significant coefficient for some year occurs often,
# even when there really is no year effect (here, due to randomization).
# Conclusion: p4t appears to be iid over stocks and years 

# fit truncated logistic distribution for p4t by maximum likelihood
dTlogis<-function(x,m,s) {dlogis(x,m,s)/(plogis(1,m,s)-plogis(0,m,s))}
pTlogis<-function(x,m,s) {(plogis(x,m,s)-plogis(0,m,s))/(plogis(1,m,s)-plogis(0,m,s))}
dTlik<-function(pars) {-sum(log(dTlogis(p4t,pars[1],pars[2])))}
sfit<-optim(c(1.5,0.2),dTlik,method="Nelder-Mead",control=list(trace=2)); 
sfit2<-optim(sfit$par,dTlik,method="Nelder-Mead",control=list(trace=0)); 
logisMLpars<-sfit2$par; 

