################################################################## 
# This file contains R code for iterating the structured         
# model, including iterations using bootstrap parameter distributions
# to account for parameter uncertainty. 		
#
# We are not at liberty to distribute the data used to fit the	
# models for Et and p4t, so users will not be able to
# run that portion of the code (which is at the top). 	
# However, at the comment line 'USERS START HERE' we insert
# 'by hand' the necessary results from the analysis so that
# the rest of the code can be run, to replicate the process
# leading to Figure 2 in the paper 
##################################################################


############## Read in the data  
H<-read.csv("e:\\projects\\UncertaintyFeature\\sdata2.csv");
e<-H$AGE4>=0; H<-H[e,];

stock<-H$STOCK;
northfork<-(stock=="Bear")|(stock=="Marsh")|(stock=="Sulphur");
H<-H[northfork,];

year<-H$YEAR;
spawners<-H$SPAWNERS; 
Et<-(H$AGE4+H$AGE5)/spawners;
p4<-H$AGE4/(H$AGE4+H$AGE5); 


############### Fit model to Et
end<- year>83&spawners>20; 
enddata<-data.frame(year=year[end], fyear=factor(year[end]),
byear=factor((year[end]==90)|(year[end]==91)),spawners=spawners[end],
Et=Et[end],stock=H$STOCK[end],p4=p4[end],recruits=H$AGE4[end]+H$AGE5[end]); 
Emodel<-lm(log(Et)~byear,data=enddata); 


############## Fit truncated logistic model to p4t
p4t<-p4[(H$AGE4+H$AGE5)>=5];

dTlogis<-function(x,m,s) {dlogis(x,m,s)/(plogis(1,m,s)-plogis(0,m,s))}
pTlogis<-function(x,m,s) {(plogis(x,m,s)-plogis(0,m,s))/(plogis(1,m,s)-plogis(0,m,s))}
dTlik<-function(pars) {-sum(log(dTlogis(p4t,pars[1],pars[2])))}
sfit<-optim(c(0.05,0.2),dTlik,method="Nelder-Mead",control=list(trace=0,parscale=c(100,100))); 
sfit2<-optim(sfit$par,dTlik,method="Nelder-Mead",control=list(trace=2,parscale=c(100,100))); 
sfit$par; sfit2$par; 
p4pars<-sfit2$par; 

y0<-Emodel$coefficients[1];
deltabad<-Emodel$coefficients[2]; 
sigmaE<-sqrt(mean(Emodel$residuals^2));

# ----------USERS START HERE --------------------------
y0=-0.4683129;  deltabad=-2.642768;  sigmaE=0.5622334;

# Create meaningless lm object to hold actual coefficients
# The functions below only use the $coefficients component
# of the model for Et, so they can use this model as if it 
# were the actual model. 
x1=runif(10); x2=runif(10); y=x1+x2+rnorm(10);
Emodel=lm(y~x1+x2); 
Emodel$coefficients=c(y0,deltabad,sigmaE); 
p4pars=c(0.05138876,0.18805830); 


#################### Function to simulate p4t values by transform-uniform method
#################### T years of n reps for 3 stocks
p4sim<-function(T,n,pars) {
	x<-c(0:50)/50; 
	m<-pars[1]; s<-pars[2]
	cdf<-(plogis(x,m,s)-plogis(0,m,s))/(plogis(1,m,s)-plogis(0,m,s))
	cdfi<-approxfun(cdf,x); 
	u<-runif(3*T*n); X<-cdfi(u);
	return( matrix(X,T,3*n) )
} 
p4t=p4sim(10,4,p4pars); p4t=matrix(p4t,40,1); 
# This creates a set of 'real data' for use in the nonparametric
# bootstrap code below. 

# last 5 stock values, and matrix of same; these data are from
# McClure et al. 2000, table VI.1 
Bear<-c(16,56,225,372,72);
Sulphur<-c(4,23,43,140,0);
Marsh<-c(0,18,110,164,0); 
Startvals<-matrix(0,5,3);
Startvals[1:5,1]<-Bear; Startvals[1:5,2]<-Sulphur;  Startvals[1:5,3]<-Marsh; 
 
################ Function to generate pseudo-Et values
Etsim<-function(T,n,model,pbad) {

y0<-model$coefficients[1];
deltabad<-model$coefficients[2]; 
sigmaE<-sqrt(mean(model$residuals^2));

Y1<-y0+sigmaE*matrix(rnorm(T*n),T,n); 
Y2<-y0+sigmaE*matrix(rnorm(T*n),T,n); 
Y3<-y0+sigmaE*matrix(rnorm(T*n),T,n);  
bad<-(matrix(runif(T*n),T,n)<pbad);
Y1[bad]<-Y1[bad]+deltabad;
Y2[bad]<-Y2[bad]+deltabad;
Y3[bad]<-Y3[bad]+deltabad;
exp(cbind(Y1,Y2,Y3))
}


# --- Do multiple iterates of fitted model to get extinction risk & PI
nreps<-5000; T<-50;

#Create population matrix
Stocks<-matrix(0,T+5,3*nreps);

# Enter initial values
Bcols<-1:nreps; Stocks[1:5,Bcols]<-Bear;
Scols<-(nreps+1):(2*nreps); Stocks[1:5,Scols]<-Sulphur;
Mcols<-(2*nreps+1):(3*nreps); Stocks[1:5,Mcols]<-Marsh;

# Generate vital rates
Etstar<-Etsim(T+5,nreps,Emodel,2/11);
p4star<-p4sim(T+5,nreps,p4pars); 
p5star<-1-p4star;

probs<-c(0.1,0.25,0.5,0.75,0.90); 
pctiles<-matrix(0,T,6);
pext<-rep(0,T);

################### iterate to generate predictions
for (i in 6:(T+5)) {
Si<-Stocks[i-5,]*Etstar[i-5,]*p5star[i-5,]
		+Stocks[i-4,]*Etstar[i-4,]*p4star[i-4,]
Si[Si<2]<-0; 
Stocks[i,]<-Si;
Totsize<-Si[Bcols]+Si[Scols]+Si[Mcols];
pctiles[(i-5),1:5]<- quantile(Totsize,probs); pctiles[(i-5),6]<-mean(Totsize);

Totlag<-Stocks[(i-4):i,Bcols] + Stocks[(i-4):i,Scols] +Stocks[(i-4):i,Mcols]; 
Totlag<-apply(Totlag,2,sum); 
pext[i-5]<-mean(Totlag==0);
}

finalsize<-Stocks[T+5,Bcols]+Stocks[T+5,Scols]+Stocks[T+5,Mcols]; 
pext0<-pext; 


# -------------------------------- Create Figure 2
par(mfrow=c(2,2),cex.axis=1.25,cex.lab=1.5,cex.main=1.35,las=1,
mar=c(5,5,3,3)); 
initsizes<- Bear[1:5]+Marsh[1:5]+Sulphur[1:5];

matplot(1:T,pctiles,type="l",col="black",xlab="",
ylab="Total #spawners",xlim=c(-4,30),ylim=c(0,750),lty=2); 
points((-4:1),c(initsizes,pctiles[1,3]),type="l",lty=1); 
points(-4:0,initsizes,type="p",pch=15); 
title("         Percentiles of #spawners"); 
mtext("a",adj=0,cex=1.8,line=0.5); 

plot(1:T,pext,ylim=c(0,1),type="l",lwd=2,xlab="",lty=1,
ylab="Probability"); title("Extinction risk"); 
mtext("b",adj=0,cex=1.8,line=0.5)

### Read bootstrap results from file and add to plot; code for the
### bootstrap is below

initsizes<- Bear[1:5]+Marsh[1:5]+Sulphur[1:5];

# Your path may differ; change this line as needed. 
bootpctiles<-read.table("e:\\projects\\uncertaintyfeature\\bootpctiles.txt"); 

matplot(1:T,bootpctiles,type="l",col="black",xlab="Years in future",
ylab="Total spawners",xlim=c(-4,30),ylim=c(0,750),lty=2); 
points((-4:1),c(initsizes,bootpctiles[1,3]),type="l",lty=1); 
points(-4:0,initsizes,type="p",pch=15); 
mtext("c",adj=0,cex=1.8,line=0.5); 

bootBCpext<-read.table("e:\\projects\\uncertaintyfeature\\bootBCpext.txt"); 
matplot(1:T,bootBCpext,ylim=c(0,1),type="l",lwd=c(2,2,2),xlab="Years in future",lty=c(1,2,2),
ylab="Probability",col="black"); 
mtext("d",adj=0,cex=1.8,line=0.5); 

# -------------Bootstrap to get limits on the extinction risk
# The number of reps used to estimate extinction risk, and 
# the number of bootstrap reps, are reduced here for the sake
# of run time. For the paper we used 
#	nreps<-250; nBoot<-2999;
 
nreps<-20; T<-50; nBoot<-199;
pexts<-matrix(0,T,nBoot);
Totsizes<-matrix(0,T,nreps*nBoot); 

# functions needed for fitting distribution of p4t 
dTlogis<-function(x,m,s) {dlogis(x,m,s)/(plogis(1,m,s)-plogis(0,m,s))}
pTlogis<-function(x,m,s) {(plogis(x,m,s)-plogis(0,m,s))/(plogis(1,m,s)-plogis(0,m,s))}

for (j in 1:nBoot) {
cat(j,"\n"); 
## generate bootstrap sample of Et and re-fit
Y<-y0+sigmaE*matrix(rnorm(33),11,3);
Y[1:2,]<-Y[1:2,]+deltabad;
fbad<-matrix(0,11,3); fbad[1:2,]<-1; 
Emodelj<-lm(Y~fbad);

y0j<-Emodelj$coefficients[1];
deltabadj<-Emodelj$coefficients[2]; 
sigmaEj<-sqrt(mean(Emodelj$residuals^2));

pvals<-c(1:999)/1000; phat<-2/11; 
qvals<-(phat-pvals)/sqrt(pvals*(1-pvals)/11); 
z<-rnorm(1); pbadj<-pvals[order(abs(z-qvals))][1]; 

## generate bootstrap sample of p4t and re-fit
p4j<-sample(p4t,replace=T); 
dTlikj<-function(pars) {-sum(log(dTlogis(p4j,pars[1],pars[2])))}
sfit<-optim(p4pars,dTlikj,method="Nelder-Mead",control=list(trace=0,parscale=c(100,100))); 
sfit<-optim(sfit$par,dTlikj,method="Nelder-Mead",control=list(trace=0,parscale=c(100,100))); 
p4parj<-sfit$par; 

#Create population matrix
Stocks<-matrix(0,T+5,3*nreps);

# Enter initial values
Bcols<-1:nreps; Stocks[1:5,Bcols]<-Bear;
Scols<-(nreps+1):(2*nreps); Stocks[1:5,Scols]<-Sulphur;
Mcols<-(2*nreps+1):(3*nreps); Stocks[1:5,Mcols]<-Marsh;

# Generate vital rates
Etstar<-Etsim(T+5,nreps,Emodelj,pbadj);
p4star<-p4sim(T+5,nreps,p4parj); 
p5star<-1-p4star;

pext<-rep(0,T);

## iterate refitted model
for (i in 6:(T+5)) {
Si<-Stocks[i-5,]*Etstar[i-5,]*p5star[i-5,]
		+Stocks[i-4,]*Etstar[i-4,]*p4star[i-4,]
Si[Si<2]<-0;
Stocks[i,]<-Si;
Totsize<-Si[Bcols]+Si[Scols]+Si[Mcols];
col1<-(j-1)*nreps+1; col2<-j*nreps;
#Totsizes[i-5,col1:col2]<-Totsize; 

Totlag<-Stocks[(i-4):i,Bcols] + Stocks[(i-4):i,Scols] +Stocks[(i-4):i,Mcols]; 
Totlag<-apply(Totlag,2,sum); 
pext[i-5]<-mean(Totlag==0); 
}

pexts[,j]<-pext;
cat(j,"\n"); 
}

lims.BC<-matrix(0,T,2); lims.pctile<-lims.BC
for (j in 1:T) {
tvals<-pexts[j,]; thetahat<-pext0[j];
p1<-quantile(tvals,0.025); p2<-quantile(tvals,0.975);
lims.pctile[j,1]<-p1; lims.pctile[j,2]<-p2; 
if (p2>p1) { 
 pwr<-1
 thetahat<-pext0[j]; z0<-qnorm(mean(pexts[j,]<thetahat));
 a1<-pnorm(2*z0+qnorm(0.05)); a2<-pnorm(2*z0+qnorm(0.95));
 lims.BC[j,1:2]<-(quantile(pexts[j,],c(a1,a2))); 
}
else {lims.BC[j,1:2]<-p1}

}

