#  Winding Stairs Algorithm for Sobol indices and calculation of confidence intervals

#  Calculation of Sobol indices, have 4 sets of parameters (eta, gamma,
#     nu, p4t for fixed harvest and release size), 

numcols<-4; # number of independent parameters sets in the model

nsims1 <- 1000; # number of parameters sets per column of winding stairs matrix
nsims2<-5000;  # number of simulations per parameter set
nyears<-20; # t = time horizon for projections

Release<-32.8;  #number of hatchery fish released (in millions) = mean of Levin data set
Harvest<-0.1;  # approx. brood year exploitation rate based on Petrosky et al. 2001

resultsN <-matrix(0,nsims1, numcols);  # to hold mu=stoch. growth rate over nyears

#  Drought indices and OCI data used for model projections
library(tseries); # needed for surrogate function

#  PDSI data to form drought indices
P2 <-read.csv("c:\\viability\\ecology_sf\\pdsi.csv");
pdsi<-P2$pdsi.fall
 
#  OCI data to form ocean index
P3<-read.table("c:\\viability\\ecology_sf\\HareMantuaOCI")
OCI<-P3[P3$year>1977,2];   #  use 1978 on since OCI appears to have declined prior to 1978
 
#---------------------------------------------Winding Stairs Algorithm for Sobol Indices-----------------------
for (i in 1:numcols){
   
#  runmodelCase() is a function that runs the stage structured simulation model and returns
#   the estimated growth rate 

for (j in 1:nsims1){ 
   epars<-etapars[j,]
   gregs<- gammaregs[j+(i>1),]
   gresids<-gammaresids[j+(i>1),]
   gyreffects<-gammayreffects[j+(i>1),]
   npars<-nupars[j+(i>2),]
   ppars<-p4pars[j+(i>3),]
 resultsN[j,i]<-runmodelCase(epars, gregs, gresids, gyreffects, npars, ppars, nyears, nsims2, Release, Harvest, 0)
}
}

save(resultsN, "c:\\viability\\ecology_sf\\baseline.txt", ascii=T)

#  Estimate total variance (each column gives independent estimate of variance) and total mean
D<-mean(apply(resultsN,2,var))  # estimate of overall variance
f0<-mean(apply(resultsN,2,mean)) # estimate of overall mean
 
 
#  Main sensitivity indices
Seta<-1-sum((resultsN[1:nsims1, 1]-resultsN[1:nsims1,4])^2)/(2*(nsims1))/D
Sgamma<-1-sum((resultsN[1:nsims1-1,2]-resultsN[2:nsims1,1])^2)/(2*(nsims1-1))/D
Snu<-1-sum((resultsN[1:nsims1-1,3]-resultsN[2:nsims1,2])^2)/(2*(nsims1-1))/D
Sp<-1-sum((resultsN[1:nsims1-1,4]-resultsN[2:nsims1,3])^2)/(2*(nsims1-1))/D
Smain<-cbind(Seta, Sgamma, Snu, Sp)

#  Total sensitivity indices
Steta<-sum((resultsN[1:nsims1-1,4]-resultsN[2:nsims1,1])^2)/(2*(nsims1-1))/D
Stgamma<- sum((resultsN[1:nsims1,1]-resultsN[1:nsims1,2])^2)/(2*(nsims1))/D
Stnu<- sum((resultsN[1:nsims1,2]-resultsN[1:nsims1,3])^2)/(2*(nsims1))/D
Stp<- sum((resultsN[1:nsims1,3]-resultsN[1:nsims1,4])^2)/(2*(nsims1))/D
Stot<-c(Steta, Stgamma, Stnu, Stp)

#---------------------------------------------Point Estimates and C.I.s----------------------------------------------

#  Obtain point estimates of mu under each scenario
ptests<-matrix(0,4,1)
Release<-32.8
Harvest<-0.1

ptests[1,]<- runmodelCase(etaparsPE, gammaregsPE, gammaresidsPE, gammayreffectsPE, nparsPE, p4parsPE, nyears, nsims2, Release, Harvest, 0)
ptests[2,]<- runmodelCase(etaparsPE, gammaregsPE, gammaresidsPE, gammayreffectsPE, nparsPE, p4parsPE, nyears, nsims2, Release, Harvest, 1)
ptests[3,]<- runmodelCase(etaparsPE, gammaregsPE, gammaresidsPE, gammayreffectsPE, nparsPE, p4parsPE, nyears, nsims2, Release, Harvest, 2)
ptests[4,]<- runmodelCase(etaparsPE, gammaregsPE, gammaresidsPE, gammayreffectsPE, nparsPE, p4parsPE, nyears, nsims2, Release, 0, 3)


colnames(ptests)<-c("mu")
rownames(ptests)<-c("Base", "A1", "A2", "A3")

print(ptests)


#  Run for nsims1 bootstrap samples to obtain confidence intervals
resultsConf<-matrix(0,nsims1,4)
for (i in 1:4){
for (j in 1:nsims1){ 
   epars<-etapars[j,]
   gregs<- gammaregs[j,]
   gresids<-gammaresids[j,]
   gyreffects<-gammayreffects[j,]
   npars<-nupars[j,]
   ppars<-p4pars[j,]

resultsConf[j,i]<-runmodelCase(epars, gregs, gresids, gyreffects, npars, ppars, nyears, nsims2, Release, Harvest, i-1)
}
}

#  Calculate BC confidence interval
bcci<-function(thetahat,thetaboot,alpha) {
# thetahat is the estimate for the actual data,
# thetaboot is a vector of estimates from bootstrap samples
# the value returned is a 100(1-alpha)% BC confidence interval
	z0<-qnorm(mean(thetaboot<thetahat));
	a1<-pnorm(2*z0+qnorm(alpha/2)); 
	a2<-pnorm(2*z0+qnorm(1-alpha/2));
	quantile(thetaboot,c(a1,a2)); 
}


#  Confidence intervals for each scenario and change from baseline
muconf<-matrix(0,4,2)
muchange<-matrix(0,4,2)

for (i in 1:4){
# confidence interval for mu
muconf[i,]<-bcci(ptests[i],resultsConf[,i],0.1); 

#confidence interval for change in mu from baseline
muchange[i,]<-bcci(ptests[i]-ptests[1], resultsConf[,i]-resultsConf[,1], 0.1);

}

print(muconf)
print(muchange)


